/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper
 * All rights reserved.
 * Modified by Tim Edwards, April, 1997 for CIO-DIO48 board
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/***************************************************************************
 *
 *  diotest.c
 *
 *  This program is used to test the CIO-DIO48
 *  Linux loadable module (dio48).
 *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/dio96.h>

/***************************************************************************
 *
 *  Global Data
 *
 ***************************************************************************/

#define LONG unsigned long
#define BYTE unsigned char

char *Dev0Name = "/dev/dio/dio0";
char *Dev1Name = "/dev/dio/dio1";

#if CHANNELS > 2
char *Dev2Name = "/dev/dio/dio2";
char *Dev3Name = "/dev/dio/dio3";
#endif

/***************************************************************************
 *
 *  Main
 *
 ***************************************************************************/

int main(int argc, char **argv)
{
  int	fd0, fd1;
#if CHANNELS > 2
  int	fd2, fd3;
#endif
  long  dataval;
  int   rval;
  char  locin[100];

  /* Open the device file for the counter */

  if ((fd0 = open(Dev0Name, O_RDWR)) < 0) {
    printf("error opening DIO96 device AP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-A to all ports output.\n");
  ioctl(fd0, AP_SET_MODE, OUTPUT);

  if ((fd1 = open(Dev1Name, O_RDWR)) < 0) {
    printf("error opening DIO96 device BP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-B to all ports output.\n");
  ioctl(fd1, BP_SET_MODE, OUTPUT);

#if CHANNELS > 2
  if ((fd2 = open(Dev2Name, O_RDWR)) < 0) {
    printf("error opening DIO96 device CP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-C to all ports output.\n");
  ioctl(fd0, CP_SET_MODE, OUTPUT);

  if ((fd3 = open(Dev3Name, O_RDWR)) < 0) {
    printf("error opening DIO96 device DP\n");
    exit(2);
  }
  printf("Setting mode of 82C55-D to all ports output.\n");
  ioctl(fd1, DP_SET_MODE, OUTPUT);
#endif

  printf("sleep before read\n");
  sleep(1);
  
    dataval = (LONG)0xffffff;
    if (write(fd0, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if ((rval = read(fd0, &dataval, 3)) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      printf("   read() returned %d, expected 3\n", rval); 
      printf("   buffer contains %0lx, expected 0xffffff\n", dataval);
      exit(1);
    }
    else printf("Data expected: 0xffffff, Data read: %0lx\n", dataval);
    sleep(1);
    dataval = (LONG)0x00;
    if (write(fd0, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if (read(fd0, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      exit(1);
    }
    else printf("Data expected: 0x000000, Data read: %0lx\n", dataval);
    sleep(1);
    ioctl(fd0, AP_SET_A, (BYTE)0xaa);
    ioctl(fd0, AP_GET_A, &dataval);
    printf("Read on A expected: 0xaa, Data read: %0lx\n", dataval);
    ioctl(fd0, AP_SET_B, (BYTE)0x55);
    ioctl(fd0, AP_GET_B, &dataval);
    printf("Read on B expected: 0x55, Data read: %0lx\n", dataval);
    ioctl(fd0, AP_SET_C, (BYTE)0xc3);
    ioctl(fd0, AP_GET_C, &dataval);
    printf("Read on C expected: 0xc3, Data read: %0lx\n", dataval);
    sleep(1);

    /* Again for device fd1 */

    dataval = (LONG)0xffffff;
    if (write(fd1, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if ((rval = read(fd1, &dataval, 3)) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      printf("   read() returned %d, expected 3\n", rval); 
      printf("   buffer contains %0lx, expected 0xffffff\n", dataval);
      exit(1);
    }
    else printf("Data expected: 0xffffff, Data read: %0lx\n", dataval);
    sleep(1);
    dataval = (LONG)0x00;
    if (write(fd1, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on write.\n");
      exit(1);
    }
    if (read(fd1, &dataval, 3) != 3 ) {
      printf("main() in test.c: error on readback.\n");
      exit(1);
    }
    else printf("Data expected: 0x000000, Data read: %0lx\n", dataval);
    sleep(1);
    ioctl(fd1, BP_SET_A, (BYTE)0xaa);
    ioctl(fd1, BP_GET_A, &dataval);
    printf("Read on A expected: 0xaa, Data read: %0lx\n", dataval);
    ioctl(fd1, BP_SET_B, (BYTE)0x55);
    ioctl(fd1, BP_GET_B, &dataval);
    printf("Read on B expected: 0x55, Data read: %0lx\n", dataval);
    ioctl(fd1, BP_SET_C, (BYTE)0xc3);
    ioctl(fd1, BP_GET_C, &dataval);
    printf("Read on C expected: 0xc3, Data read: %0lx\n", dataval);
    sleep(1);

  printf("Bitwise test: \n");
  printf("Pin numbers designated as {device}P{channel}{pin} as per\n");
  printf("the National Instruments DIO-96 manual pages 2-10 and 2-11.\n");
  strcpy(locin, "APC7");
  for(;;){
     char linein[100], *newline;
     char devi, chi;
     int pinn;

     printf("Enter pin number to toggle or Q to quit [%s]: ", locin);
     fgets(linein, 100, stdin);
     if ((newline = strrchr(linein, '\n')) != NULL) *newline = '\0';
     if (strlen(linein) == 0) strcpy(linein, locin);

     if (sscanf(linein, "%c%*c%c%d", &devi, &chi, &pinn) != 3) {
	if (devi == 'Q' || devi == 'q') break;
	printf("Error in format: example--- APA0, APB5, BPC6. . ."); 
     }
     else if (pinn < 0 || pinn > 7) {
	printf("Pin number out of range 0 to 7\n");
     }
     else {
	devi = toupper(devi);
	chi = toupper(chi);
        devi -= 'A';
	chi -= 'A';
	if (chi >= 0 && chi <= 3 && devi >= 0 && devi <= 4) {
	   ioctl(fd0, devi << 8 | chi << 3 | 0x60 | pinn, &dataval);
	   dataval = 1 - (dataval & 0x01);
	   if (dataval) { 
	      ioctl(fd0, devi << 8 | chi << 3 | 0xe0 | pinn);
	      printf("H");
	   }
	   else {
	      ioctl(fd0, devi << 8 | chi << 3 | 0x80 | pinn);
	      printf("L");
	   }
	   strcpy(locin, linein);
	}
	else {
	   if (devi < 0 || devi > 4) printf("Device out of range A-D");
	   if (chi < 0 || chi > 3) printf("Channel out of range A-C");
	}
     }
     printf("\n");

  }

  printf("Setting all outputs to zero\n");
  dataval = (LONG)0x00;
  write(fd0, &dataval, 3);
  write(fd1, &dataval, 3);
#if CHANNELS > 2
  write(fd2, &dataval, 3);
  write(fd3, &dataval, 3);
#endif

  printf("Exiting.\n");
  close(fd0);
  close(fd1);
#if CHANNELS > 2
  close(fd2);
  close(fd3);
#endif
  return 0;
}
